#pragma comment(lib, "dxguid.lib")
#pragma comment(lib, "d3d9.lib")
#pragma comment(lib, "d3dx9.lib")

#include <tchar.h>
#include <d3d9.h>
#include <d3dx9.h>

#include "stdafx.h"
#include <windows.h>
#include <fstream>

#include "oxjson.h"

using namespace OX;


void* getValue( void *p, int index, DWORD stride, int offset ) {
    return (void*)((char*)p + stride * index + offset);
}

template< class T, class INDEXBUFFERTYPE>
void getValues( std::vector<T> &ary, void *pVb, void *pIb, DWORD indexNum, int stride, int offset, int dim ) {
	for ( DWORD i = 0; i < indexNum; i++ ) {
		T* v = (T*)getValue( pVb, ((INDEXBUFFERTYPE*)pIb)[i], stride, offset );
		for ( int j = 0; j < dim; j++ )
			ary.push_back( v[j] );
	}
}

int getOffsetFromElem( D3DVERTEXELEMENT9 *elems, D3DDECLUSAGE usage ) {
    D3DVERTEXELEMENT9 *e = elems;
    while( e->Stream != 0xff ) {
        if ( e->Usage == usage )
            return e->Offset;
        e = e + 1;
    }
    return -1;
}

template< class T, int dim >
Hash getElementHash( ID3DXMesh *mesh , D3DDECLUSAGE usage ) {
    // _錾擾vf̃ItZbgƃXgChTCY擾
    D3DVERTEXELEMENT9 e[MAX_FVF_DECL_SIZE];
    mesh->GetDeclaration( e );
    int offset = getOffsetFromElem( e, usage );
	if ( offset == -1 )
		return Hash();
    DWORD stride = mesh->GetNumBytesPerVertex();

    IDirect3DVertexBuffer9 *vb;
    IDirect3DIndexBuffer9 *ib;
	mesh->GetVertexBuffer( &vb );
	mesh->GetIndexBuffer( &ib );

	void *pVb = 0;
    void *pIb = 0;
    if ( FAILED( vb->Lock(0, 0, &pVb, 0) ) )
        return Hash();
    if ( FAILED( ib->Lock(0, 0, &pIb, 0) ) ) {
        vb->Unlock();
        return Hash();
	}

	D3DINDEXBUFFER_DESC desc;
	ib->GetDesc( &desc );

    std::vector<T> ary;
    DWORD indexNum = mesh->GetNumFaces() * 3;
	if ( desc.Format == D3DFMT_INDEX16 )
		getValues<T, WORD>( ary, pVb, pIb, indexNum, stride, offset, dim );
	else
		getValues<T, DWORD>( ary, pVb, pIb, indexNum, stride, offset, dim );

	vb->Unlock();
    ib->Unlock();

	vb->Release();
	ib->Release();

    Hash outHash;
    outHash["data"] = ary;
    outHash["dim"] = dim;

    return outHash;
}

Hash createMeshHashFromX( IDirect3DDevice9 *dev, TCHAR *fileName ) {
	ID3DXMesh *mesh = 0;
	ID3DXBuffer *materialsBuf = 0;
	DWORD numMaterials = 0;
	if ( FAILED( D3DXLoadMeshFromX( fileName, D3DXMESH_MANAGED, dev, 0, &materialsBuf, 0, &numMaterials, &mesh ) ) )
		return Hash();

	Hash vertexHash;
	vertexHash["coord"] = getElementHash<float, 3>( mesh, D3DDECLUSAGE_POSITION );
	vertexHash["color"] = getElementHash<float, 4>( mesh, D3DDECLUSAGE_COLOR );

	Hash meshHash;
	meshHash["vertex"] = vertexHash;
	meshHash["primitive"] = "triangle";
	meshHash["vertexnum"] = (int)mesh->GetNumFaces() * 3;

	materialsBuf->Release();
	mesh->Release();

	return meshHash;
}

int _tmain(int argc, _TCHAR* argv[]) {

	if ( argc < 2 ) {
		printf("meshconverter.exe [XfilePath] [outputFilePath]");
		return -1;
	}

	HWND hWnd = ::GetDesktopWindow();

	// Direct3D̏
	LPDIRECT3D9 g_pD3D;
	LPDIRECT3DDEVICE9 g_pD3DDev;
	if( !(g_pD3D = Direct3DCreate9( D3D_SDK_VERSION )) ) return 0;
	D3DPRESENT_PARAMETERS d3dpp = {0,0,D3DFMT_UNKNOWN,0,D3DMULTISAMPLE_NONE,0, D3DSWAPEFFECT_DISCARD,NULL,TRUE,0,D3DFMT_UNKNOWN,0,0}; 
	if( FAILED( g_pD3D->CreateDevice( D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, hWnd, D3DCREATE_HARDWARE_VERTEXPROCESSING, &d3dpp, &g_pD3DDev ) ) ) {
		g_pD3D->Release();
		return -1;
	}

	Hash rootHash = createMeshHashFromX( g_pD3DDev, argv[1] );

	// o
	std::string outStr = OX::Json::encode( rootHash );
	std::ofstream ofs( argv[2] );
	ofs.write( outStr.c_str(), outStr.size() );

	g_pD3DDev->Release();
	g_pD3D->Release();
}
