#ifndef IKD_OX_BASIC_OXSP_H
#define IKD_OX_BASIC_OXSP_H

// X}[g|C^

#define OXNEW new
#define OXDELETE(x) (delete x)

	struct Ref {
		unsigned ref, wref;
		Ref( unsigned ref ) : ref(ref), wref() {}
		Ref( unsigned ref, unsigned wref ) : ref(ref), wref(wref) {}
	};

namespace OX {
	template <class T> class wp;

	// X}[g|C^
	template <class T>
	class sp {
		friend class wp<T>;
	private:
		void release() {
			if (ref) {
				--ref->ref; --ref->wref;
				if (ref->ref == 0)
					OXDELETE(ptr);
				if (ref->wref == 0)
					OXDELETE(ref);
			} 
		}
		void copy(const sp<T> &r) {
			ptr = r.getPtr(); ref = r.ref;
			if (ref) {
				++ref->ref;
				++ref->wref;
			}
		}
		template<class T2>
		void copy(sp<T2> &r) {
			ptr = r.getPtr(); ref = r.getRefPtr();
			if (ref) {
				++ref->ref;
				++ref->wref;
			}
		}

		T* ptr;
		Ref *ref;

	public:
		sp() : ptr(), ref() {}
		sp(T* src) : ptr(src), ref() { ref = ptr ? OXNEW Ref(1, 1) : 0; }
		sp(const sp &r) { copy(r); }
		template< class T2 >
		sp(const sp<T2> &r) { copy(r); }
		sp(T* src, Ref* ref) : ptr(src), ref(ref) {}
		~sp() { release(); }

		unsigned refCount() const { return ref ? ref->ref : 0; }
		Ref *getRefPtr() { return ref; }
		T & operator *() const { return *ptr; }
		T* operator ->() const { return ptr; }
		T* getPtr() const { return ptr; }
		template <class T2>
		sp<T2> cast() {
			T2 *cast_ptr = dynamic_cast<T2*>(ptr);
			if (ref) {
				++ref->ref; ++ref->wref;
				return sp<T2>(cast_ptr, ref);
			}
			return sp<T2>();
		}
		void operator =(const unsigned &r) {
			release();
			ptr = 0;
			ref = 0;
		}
		sp<T> & operator =(const sp<T> &r) {
			if (this != &r) {
				release();
				copy(r);
			}
			return *this;
		}
		template <class T2>
		sp<T> & operator =(sp<T2> &r) {
			release();
			copy(r);
			return *this;
		}
	};


	// EB[N|C^
	template <class T>
	class wp {
		T* ptr;
		Ref *ref;

		void release() {
			if (ref && --(ref->wref) == 0)
				OXDELETE(ref);
		}
		void copy(sp<T> &r) {
			ptr = r.getPtr();
			ref = r.ref;
			if (ref)
				++ref->wref;
		}
		template<class T2>
		void copy(sp<T2> &r) {
			ptr = r.getPtr();
			ref = r.getRefPtr();
			if (ref)
				++ref->wref;
		}
		void copy(wp<T> &r) {
			ptr = r.getPtr();
			ref = r.ref;
			if (ref)
				++ref->wref;
		}
		template<class T2>
		void copy(wp<T2> &r) {
			ptr = r.getPtr();
			ref = r.getRefPtr();
			if (ref)
				++ref->wref;
		}

	public:
		wp() : ptr(), ref() {}
		wp(sp<T> &spR) { copy(spR); }
		wp(wp<T> &wpR) { copy(wpR); }
		template <class T2>
		wp(sp<T2> &spR) { copy(spR); }
		template <class T2>
		wp(wp<T2> &wpR) { copy(wpR); }
		~wp() { release(); }
		unsigned refCount() const { return ref ? ref->ref : 0; }
		unsigned weakRefCount() const { return ref ? ref->wref : 0; }
		Ref *getRefPtr() { return ref; }
		T & operator *() const { return *ptr; }
		T* operator ->() const { return ptr; }
		T* getPtr() const { return ptr; }
		sp<T> toSP() {
			if (ref && ref->ref) {
				++ref->ref; ++ref->wref;
				return sp<T>( ptr, ref );
			}
			return sp<T>();
		}
		bool isExist() const {
			return (ref && (ref->ref > 0));
		}
		wp<T> & operator =(sp<T> &r) {
			release();
			copy(r);
			return *this;
		}
		template <class T2>
		wp<T> & operator =(sp<T2> &r) {
			release();
			copy(r);
			return *this;
		}
		wp<T> & operator =(wp<T> &r) {
			release();
			copy(r);
			return *this;
		}
		template <class T2>
		wp<T> & operator =(wp<T2> &r) {
			release();
			copy(r);
			return *this;
		}
	};
}

#endif
